<%inherit file="base.html" />

<%block name="title">${ conf.sitename } – Artikelübersicht</%block>

<%block name="content">
<article>
  <table id="arc">
  <% year = -1 %>
  % for key, months in sorted(articles.iteritems(), reverse=True):
    <tr>
        % if year != key[0]:
          <% year = key[0] %>
          <th>${ str(key[0]) + ' ' + months[0]['date'].strftime('%B') }</th>
        % else:
          <th>${ months[0]['date'].strftime('%B') }</th>
        % endif
        <td></td>
    </tr>
    % for entry in months:
      <tr><th>${ entry.date.day }</th>
          <td><a href="${ entry.permalink }">${ entry.title }</a></td>
      </tr>
    % endfor
  % endfor
  </table>
</article>
</%block>
