# -*- encoding: utf-8 -*-
#
# Copyright 2012 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid/__init__.py


class Language(object):
    pass


class DE(Language):

    older_posts = u''
    newer_posts = u''
