html{
  height: 100%;
}

body {
  font-family: "lucida sans", arial, sans-serif;
  padding: 0px;
  margin: 0px;
  font-size: .78em;
  background: #F6F4F0;
  color: #635B53;
}

p {
  margin: 0px;
  padding: 0px 0px 16px 0px;
  line-height: 1.7em;
}

h1, h2, h3, h4 {
  font-size: 150%;
  padding: 7px 0px 5px 0px;
  margin: 0px 0px 12px 0px;
  font-weight: normal;
  color: #342F28;
  font-family: serif;
}

h1, h3 {
  text-transform: uppercase;
  margin: 10px 0px 20px 0px;
  padding: 5px 20px 5px 0px;
  border-bottom: 1px solid #CCC6B4;
}

a, a:hover { color: #635B53;
  text-decoration: none;
}

a:hover {
  color: #A4AA04;
}

img {
  border: 0px;
  margin: 0px;
  padding: 0px;
}

a {
  outline: none;
}

.left {
  float: left;
  margin-right: 10px;
}

.right {
  float: right;
  margin-left: 10px;
}

.center {
  display: block;
  text-align: center;
  margin: 0 auto;
}

blockquote {
  margin: 20px 0px 20px 0px;
  padding: 10px 20px 0px 20px;
  border: 1px solid #E5E5DB;
  border-left: 5px solid #B8BD20 !important;
  background: #FFF;
}

ul {
  margin: 2px 0px 18px 16px;
  padding: 0px;
}

ul li {
  list-style-type: square;
  margin: 0px 0px 6px 0px;
  padding: 0px;
}

ol {
  margin: 8px 0px 0px 24px;
  padding: 0px;
}

ol li {
  margin: 0px 0px 11px 0px;
  padding: 0px;
}

#main, #logo, nav, #site_content, footer {
  margin-left: auto;
  margin-right: auto;
}

body > div > header {
  background: url(img/back.png) repeat-x;
}

#logo {
  width: 870px;
  height: 127px;
  position: relative;
  background: transparent url(img/logo.png) no-repeat right;
}

#logo #logo_text {
  position: absolute;
  top: 0;
  left: 0;
}

#logo h1,#logo h2 {
  font: normal 300% 'century gothic',arial,sans-serif;
  border-bottom: 0;
  text-transform: none;
  margin: 0 0 0 9px;
}

#logo_text h1,#logo_text h1 a,#logo_text h1 a:hover {
  padding: 22px 0 0 0;
  color: #777;
  letter-spacing: 0.1em;
  text-decoration: none;
}

#logo_text h1 a .logo_colour {
  color: #B8BD20;
}

#logo_text a:hover .logo_colour {
  color: #777;
}

#logo_text h2 {
  font-size: 120%;
  padding: 4px 0 0 0;
  color: #BBB;
}

#logo h1,#logo h2 {
  margin: 0;
  font-family: "century gothic",arial;
}

#logo h1 {
  font-size: 300%;
  padding: 22px 0px 0px 0px;
  color: #777;
}

#logo h2 {
  font-size: 120%;
  padding: 4px 0px 0px 0px;
  color: #BBB;
}

#logo a {
  border: none;
}

nav {
  width: 870px;
  height: 46px;
  padding-top: 4px;
  padding-left: 4px;
}

nav ul {
  float: right;
  margin: 0px;
}

nav ul li {
  float: left;
  margin: 5px 0 0 0;
  padding: 0 0 0 6px;
  list-style: none;
}

nav ul li a {
  letter-spacing: 0.1em;
  font-family: serif;
  font-size: 100%;
  display: block;
  float: left;
  height: 20px;
  text-decoration: none;
  padding: 9px 22px 5px 16px;
  text-align: center;
  color: #FFF;
  text-transform: uppercase;
  border: none;
}

nav ul li.selected a {
  height: 20px;
  padding: 7px 22px 4px 16px;
}

nav ul li.selected {
  margin: 7px 0 0 0;
  background: #635B53 url(img/tabs.png) no-repeat 0 0px;
}

nav ul li.selected a,ul nav li.selected a:hover {
  background: #635B53 url(img/tabs.png) no-repeat 100% 0px;
  color: #FFF;
}

nav ul li a:hover {
  color: #A4AA04;
}

#site_content {
  width: 875px;
  margin-top: 10px;
  overflow: hidden;
}

.sidebar {
  float: right;
  width: 178px;
  padding: 20px 25px 15px 15px;
  min-height: 400px;
}

.sidebar ul {
  width: 178px;
  padding-top: 4px;
  margin: 4px 0px 30px 0px;
}

.sidebar li {
  list-style: none;
  padding: 0px 0px 2px 0px;
}

.sidebar li a,.sidebar li a:hover {
  text-decoration: none;
  padding: 4px 0px 4px 35px;
  display: block;
  background: transparent url(img/arrow.png) no-repeat left center;
  color: #555;
  border: none;
}

.sidebar li a.selected,.sidebar li a:hover,.sidebar li a.selected:hover {
  color: #A4AA04;
}

#content {
  text-align: justify;
  width: 624px;
  float: left;
  padding: 20px 10px 15px 0px;
}

#content a {
  color: #A4AA04;
  text-decoration: none;
}

#content img {
  border: none;
  max-width: 400px;
}

body > div > footer {
  width: 100%;
  font-family: sans-serif;
  font-size: 90%;
  height: 80px;
  padding: 20px 0px 4px 0px;
  margin-bottom: 20px;
  text-align: center;
  background: #313131;
  color: #A8AA94;
  border-top: 4px solid #A4AA04;
  text-transform: uppercase;
  letter-spacing: 0.1em;
}

body > div > footer a {
  color: #A8AA94;
  text-decoration: none;
}

body > div > footer a:hover {
  color: #FFF;
  text-decoration: none;
}

.separator {
  width: 100%;
  height: 0;
  border-top: 1px solid #D9D5CF;
  border-bottom: 1px solid #FFF;
  margin: 0 0 20px 0;
}

article > header time {
  padding: 5px 0px 5px 0px;
  float: right;
  clear: both;
}

table {
  margin: 10px 0 30px 0;
}

#arc {
  border-collapse: collapse;
}

#arc th,#arc td {
  padding-top: 0.4em;
  list-style: none;
  margin: 0;
}

#arc th {
  text-align: right;
  padding: 0.4em 1.75em 0 0;
  vertical-align: baseline;
}

.floatright {
  float: right;
}

.floatleft {
  float: left;
}